<?php

include __DIR__ . '/Curl.php';

/** Expay class which provides all communication with Expay API */
class ExpayProvider {
  private $url;
  private $public_key;
  private $private_key;

  public function __construct($settings) {
    $this->public_key = $settings['public_key'];
    $this->private_key = $settings['private_key'];
    $this->url = $settings['mode'] === 'test' ? $settings['sandbox_link'] : $settings['live_link'];
  }

  private function hashData($method, $params) {
    $string_for_hash = $method . http_build_query($params, null, '&', PHP_QUERY_RFC3986);
    return hash_hmac('sha1', $string_for_hash, $this->private_key);
  }

  public function hashResponse($response) {
    return hash_hmac('sha1', $response, $this->private_key);
  }

  public function getMethodList() {
    if ($cache = cache_get('expay_method_list')) {
      return $cache->data;
    }

    $getMethodsURL = $this->url . 'getMethods';
    $params = [
      'key' => $this->public_key,
      'timestamp' => time(),
    ];
    $params['hash'] = $this->hashData('getMethods?', $params);

    $ch = new Curl();
    $result = $ch->post($getMethodsURL, $params);

    if (
      isset($result['error']) ||
      !empty($result['curl_error']) ||
      isset($result['body']['error']) ||
      $result['http_code'] === 502
    ) {
      watchdog(
        'EPS Provider',
        'Error response of getMethodList',
        null,
        WATCHDOG_ALERT
      );
      return null;
    }
    cache_set('expay_method_list', $result['body']['response']['methods'], 'cache', time() + 86400);
    return $result['body']['response']['methods'];
  }

  public function getServicesCategories() {
    if ($cache = cache_get('expay_categories')) {
      return $cache->data;
    }

    $getServicesURL = $this->url . "getCategories";

    $params = [
      'key' => $this->public_key,
      'timestamp' => time(),
    ];
    $params['hash'] = $this->hashData('getCategories?', $params);

    $ch = new Curl();
    $result = $ch->post($getServicesURL, $params);

    if (
      isset($result['error']) ||
      !empty($result['curl_error']) ||
      isset($result['body']['error']) ||
      $result['http_code'] === 502
    ) {
      watchdog(
        'EPS Provider',
        'Error response of getServicesCategories',
        null,
        WATCHDOG_ALERT
      );
      return null;
    }
    $categories = [];
    foreach ($result['body']['response']['categories'] as $category) {
      $categories[$category['key']] = $category;
    }
    cache_set('expay_categories', $categories, 'cache', time() + 86400);
    return $categories;
  }

  public function initPayment($service_id, $amount, $order, $url_success, $url_waiting, $url_reject, $attrs = [])  {
    $initPaymentURL = $this->url . 'initPayment?';
    $ch = new Curl();

    $params = [
      'service_id' => $service_id,
      'amount' => $amount,
      'order' => $order,
      '_successUrl' => $url_success,
      '_waitingUrl' => $url_waiting,
      '_rejectUrl' => $url_reject,
      'key' => $this->public_key,
      'timestamp' => time()
    ];

    foreach ($attrs as $key => $value) {
      $params[$key] = $value;
    }

    $params['hash'] = $this->hashData('initPayment?', $params);

    $result = $ch->post($initPaymentURL, $params);
    if (isset($result['error']) || !empty($result['curl_error'])) {
      watchdog(
        'EPS Provider',
        'Error response of initPayment',
        null,
        WATCHDOG_ALERT
      );
      return null;
    }

    if (array_key_exists('error', $result['body'])) {
      watchdog(
        'EPS Provider',
        'Error response of initPayment (' . $result['body']['error']['message'] . ').',
        null,
        WATCHDOG_ALERT
      );
      return $result['body'];
    }

    return $result['body']['response'];
  }

  public function getPaymentStatus($payment_id) {
    $getStatusURL = $this->url . 'getStatus?';
    $params = [
      'payment_id' => $payment_id,
      'key' => $this->public_key,
      'timestamp' => time(),
    ];
    $params['hash'] = $this->hashData('getStatus?', $params);

    $ch = new Curl();
    $result = $ch->post($getStatusURL, $params);

    if (isset($result['error']) || !empty($result['curl_error'])) {
      watchdog(
        'EPS Provider',
        'Error response of getPaymentStatus',
        null,
        WATCHDOG_ALERT
      );
      return null;
    }

    if (isset($result['body']['error'])) {
      watchdog(
        'EPS Provider',
        'Error response of getPaymentStatus (' . $result['body']['error']['message'] . ').',
        null,
        WATCHDOG_ALERT
      );
      return null;
    }
    return $result['body']['response']['status'];
  }
}

